import { type TransformerUIProps } from '@grafana/data';
import { type SeriesToRowsTransformerOptions } from '@grafana/data/internal';
export const SeriesToRowsTransformerEditor = ({
  input,
  options,
  onChange,
}: TransformerUIProps<SeriesToRowsTransformerOptions>) => {
  return null;
};
