"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[5490,9604],{58962(l,d,o){o.d(d,{g:()=>M});var a=o(14553),r=o(55501),n=o(52358),E=o(31612),i=o(13400);const s=(0,r.lazy)(()=>Promise.all([o.e(3720),o.e(5874),o.e(715)]).then(o.bind(o,14167)).then(t=>({default:t.CodeEditor})));function M(t){return(0,a.jsx)(E.Xw,{boundaryName:"CodeMirrorEditorLazy",title:(0,n.t)("grafana-ui.code-mirror.error-label","CodeMirror editor failed to load"),style:"page",children:(0,a.jsx)(r.Suspense,{fallback:(0,a.jsx)(i._,{text:(0,n.t)("grafana-ui.code-mirror.loading-placeholder","Loading editor")}),children:(0,a.jsx)(s,{...t})})})}},70760(l,d,o){o.r(d),o.d(d,{CodeMirrorEditor:()=>r.g,TableNG:()=>n.i,attachSkeleton:()=>a.j,skeletonAnimation:()=>a.Q});var a=o(92723),r=o(58962),n=o(37943)},70783(){}}]);

//# sourceMappingURL=5490-react19.eff3e3a93332099f1811.js.map