"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[2513],{22904(E,n,a){a.r(n),a.d(n,{plugin:()=>D});var u=a(48041),s=a(83288);const D=new u.tD(s.N8)}}]);

//# sourceMappingURL=mixedPlugin.46c5b11a3661fa468e29.js.map