"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[2664,9604],{79939(f,h,e){e.d(h,{J:()=>u});var o=e(52358),a=e(28520),i=e(55816),c=e(14553),m=e(23427),d=e(29798);const r=({onChange:g,value:l,id:p})=>(0,c.jsx)(d.d,{id:p,label:m.Tp.components.PanelEditor.OptionsPane.fieldLabel("Enable pagination"),value:!!l,onChange:b=>{g(b?.currentTarget.checked)}}),u=g=>{const l=[(0,o.t)("table.category-table","Table")];g.addBooleanSwitch({path:"showHeader",name:(0,o.t)("table.name-show-table-header","Show table header"),category:l,defaultValue:i.K.showHeader}).addNumberInput({path:"frozenColumns.left",name:(0,o.t)("table.name-frozen-columns","Frozen columns"),description:(0,o.t)("table.description-frozen-columns","Columns are frozen from the left side of the table"),settings:{placeholder:(0,o.t)("table.placeholder-frozen-columns","none")},category:l}).addRadio({path:"cellHeight",name:(0,o.t)("table.name-cell-height","Cell height"),category:l,defaultValue:i.K.cellHeight,settings:{options:[{value:a.qM.Sm,label:(0,o.t)("table.cell-height-options.label-small","Small")},{value:a.qM.Md,label:(0,o.t)("table.cell-height-options.label-medium","Medium")},{value:a.qM.Lg,label:(0,o.t)("table.cell-height-options.label-large","Large")}]}}).addNumberInput({path:"maxRowHeight",name:(0,o.t)("table.name-max-height","Max row height"),category:l,settings:{placeholder:(0,o.t)("table.placeholder-max-height","none"),min:0}}).addCustomEditor({id:"enablePagination",path:"enablePagination",name:(0,o.t)("table.name-enable-pagination","Enable pagination"),category:l,editor:r,defaultValue:i.K?.enablePagination})}},80559(f,h,e){e.r(h),e.d(h,{plugin:()=>b});var o=e(45468),a=e(52358),i=e(79939),c=e(72178),m=e(15440),d=e(49441),r=e(15283),u=e(22662),g=e(64702);function l(n){if(n.hasPreferredVisualisationType("logs")||n.hasDataFrameType(u.m.LogLines))return r.nQ.Best}const p=n=>{if(!g.$.featureToggles.logsTablePanelNG)return;const s=l(n);if(s!==void 0)return[{score:s,cardOptions:{previewModifier:t=>{t.options&&(t.options.showHeader=!1,t.options.disableKeyboardEvents=!0),t.fieldConfig&&t.fieldConfig.defaults.custom&&(t.fieldConfig.defaults.custom.minWidth=50)}}}]},b=new o.m(c.O).useFieldConfig(m.K).setPanelOptions(n=>{(0,i.J)(n);const s=[(0,a.t)("logstable.category-table","Logs Table")];n.addBooleanSwitch({path:"enableLogDetails",name:(0,a.t)("logstable.enable-log-details.name","Enable log details"),category:s,description:(0,a.t)("logstable.enable-log-details.description","When enabled, shows log details for each row."),defaultValue:d.K.enableLogDetails}).addBooleanSwitch({path:"showCopyLogLink",name:(0,a.t)("logstable.show-copy-log.name","Show copy log link button"),category:s,description:(0,a.t)("logstable.show-copy-log.description","Enables/disables the log line link button in the first column of each row"),defaultValue:d.K.showCopyLogLink}).addBooleanSwitch({path:"showControls",name:(0,a.t)("logs.name-show-controls","Show controls"),category:s,description:(0,a.t)("logstable.description-show-controls","Display table controls"),defaultValue:d.K.showControls??!1}).addBooleanSwitch({path:"allowDownload",name:(0,a.t)("logs.name-allow-download","Display download control"),category:s,description:(0,a.t)("logs.description-allow-download","When controls are enabled, show an option to download the logs on display."),showIf:t=>!!t.showControls})}).setSuggestionsSupplier(p)},70783(){}}]);

//# sourceMappingURL=logsTablePanel-react19.33d264c2795110327475.js.map