"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[6857],{86029(m,i,a){a.r(i),a.d(i,{default:()=>d});var t=a(14553),e=a(52358),s=a(45270),o=a(74144),_=a(68763),E=a(94564),r=a(84285),l=a(32658),P=a(30465),g=a(77628),D=a(25877),O=a(75325),M=a(60699);function c(){const n=(0,r.Fm)({action:l.mF.Create});return(0,t.jsxs)(_.B,{direction:"column",gap:1,children:[(0,t.jsxs)(_.B,{direction:"row",alignItems:"center",justifyContent:"space-between",children:[(0,t.jsx)(o.E,{variant:"body",color:"secondary",children:(0,t.jsx)(e.x6,{i18nKey:"alerting.templates-page.description",children:"Create notification templates to customize your notifications."})}),(0,E.TT)(n)&&(0,t.jsx)(s.z9,{icon:"plus",variant:"primary",href:"/alerting/notifications/templates/new",disabled:!n.granted,children:(0,t.jsx)(e.x6,{i18nKey:"alerting.templates-page.add-template",children:"New notification template"})})]}),(0,t.jsx)(M.v,{})]})}function f(){const{navId:n,pageNav:T}=(0,P.yy)();return(0,t.jsxs)(D.y,{navId:n,pageNav:T,accessType:"notification",children:[(0,t.jsx)(O.e,{currentAlertmanager:"grafana"}),(0,t.jsx)(c,{})]})}const d=(0,g.S)(f)}}]);

//# sourceMappingURL=TemplatesPage-react19.8a511f93e2d6827f1362.js.map