"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[5334],{35334(v,_,a){a.d(_,{ConcatenateTransformerEditor:()=>m});var n=a(74848),c=a(96540),e=a(69676),r=a(52358),o=a(72480),i=a(87748),E=a(16244);const m=(0,c.memo)(({options:l,onChange:d})=>{const M=t=>{const b=t.value??e.x1.FieldName;d({...l,frameNameMode:b})},h=t=>{d({...l,frameNameLabel:t.target.value})},s=[{value:e.x1.FieldName,label:(0,r.t)("transformers.concatenate-transformer-editor.name-modes.label.copy-frame-name-to-field","Copy frame name to field name")},{value:e.x1.Label,label:(0,r.t)("transformers.concatenate-transformer-editor.name-modes.label.label-frame","Add a label with the frame name")},{value:e.x1.Drop,label:(0,r.t)("transformers.concatenate-transformer-editor.name-modes.label.ignore-the-frame-name","Ignore the frame name")}],f=l.frameNameMode??e.x1.FieldName;return(0,n.jsxs)("div",{children:[(0,n.jsx)(o.I,{label:(0,r.t)("transformers.concatenate-transformer-editor.label-name","Name"),labelWidth:16,grow:!0,children:(0,n.jsx)(i.l6,{width:36,options:s,value:s.find(t=>t.value===f),onChange:M})}),f===e.x1.Label&&(0,n.jsx)(o.I,{label:(0,r.t)("transformers.concatenate-transformer-editor.label-label","Label"),labelWidth:16,grow:!0,children:(0,n.jsx)(E.p,{width:36,value:l.frameNameLabel??"",placeholder:(0,r.t)("transformers.concatenate-transformer-editor.placeholder-frame","Frame"),onChange:h})})]})});m.displayName="ConcatenateTransformerEditor"}}]);

//# sourceMappingURL=5334.6dcbc5da08ae56c313b5.js.map