"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[558],{46755(U,i,t){t.r(i),t.d(i,{default:()=>f});var a=t(74848),E=t(54148),r=t(52358),o=t(64702),g=t(35835),P=t(32210),l=t(13425),m=t(30465),M=t(71897),u=t(93621),v=t(77628),D=t(25877),O=t(65180);const d=()=>{const[e]=(0,P.l)(),{selectedAlertmanager:_}=(0,M.VI)(),n=e.get("muteName"),{isLoading:A,data:s,isError:C}=(0,g.zm)({alertmanager:_,name:n});if(!n)return(0,a.jsx)(E.C5,{replace:!0,to:"/alerting/routes"});const I=s?.metadata?.annotations?.[l.T.Provenance];return(0,a.jsx)(O.A,{editMode:!0,loading:A,showError:C,muteTiming:s,provenance:I})};function T(){const{navId:e}=(0,m.s4)(),_=o.$.featureToggles.alertingNavigationV2,n=(0,u.aU)(_);return(0,a.jsx)(D.y,{navId:e,pageNav:{id:"alert-policy-edit",text:(0,r.t)("alerting.edit-mute-timing-page.text.edit-time-interval","Edit time interval"),parentItem:{text:(0,r.t)("alerting.time-intervals.title","Time intervals"),url:n}},accessType:"notification",children:(0,a.jsx)(d,{})})}const f=(0,v.S)(T)}}]);

//# sourceMappingURL=EditMuteTiming.64a6fa04c97d394e38ba.js.map