"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[221],{20221(r,e,a){a.r(e),a.d(e,{default:()=>t});var t={"grafana-scenes":{components:{"adhoc-filter-pill":{"edit-filter-with-key":"Redigera filter med nyckeln {{keyLabel}}","managed-filter":"Filter som hanteras av {{origin}}","non-applicable":"","remove-filter-with-key":"Ta bort filter med nyckeln {{keyLabel}}"},"adhoc-filters-combobox":{"remove-filter-value":"Ta bort filterv\xE4rde \u2013 {{itemLabel}}","use-custom-value":"Anv\xE4nd anpassat v\xE4rde: {{itemLabel}}"},"fallback-page":{content:"Om du kom hit via en l\xE4nk kan det finnas en bugg i den h\xE4r applikationen.",subTitle:"Webbadressen matchade ingen sida",title:"Hittades inte"},"lazy-loader":{placeholder:""},"nested-scene-renderer":{"collapse-button-label":"D\xF6lj scen","expand-button-label":"Visa scen","remove-button-label":"Ta bort scen"},"scene-debugger":{"object-details":"Information om objekt","scene-graph":"Scengraf","title-scene-debugger":"Scenfels\xF6kare"},"scene-grid-row":{"collapse-row":"D\xF6lj rad","expand-row":"Expandera rad"},"scene-refresh-picker":{"text-cancel":"Avbryt","text-refresh":"Uppdatera","tooltip-cancel":""},"scene-time-range-compare-renderer":{"button-label":"J\xE4mf\xF6relse","button-tooltip":"Aktivera j\xE4mf\xF6relse av tidsram"},splitter:{"aria-label-pane-resize-widget":"Widget f\xF6r storleks\xE4ndring av ruta"},"time-picker":{"move-backward-tooltip":"","move-forward-tooltip":""},"viz-panel":{title:{title:"Titel"}},"viz-panel-explore-button":{explore:"Utforska"},"viz-panel-renderer":{"loading-plugin-panel":"L\xE4ser in till\xE4ggspanel\xA0\u2026","panel-plugin-has-no-panel-component":"Paneltill\xE4gg har ingen panelkomponent"},"viz-panel-series-limit":{"content-rendering-series-single-panel-impact-performance":"Att \xE5terge f\xF6r m\xE5nga serier i en enda panel kan p\xE5verka prestandan och g\xF6ra data sv\xE5rare att l\xE4sa. ","warning-message":"Visar endast {{seriesLimit}} serier"}},utils:{"controls-label":{"tooltip-remove":"Ta bort"},"loading-indicator":{"content-cancel-query":"Avbryt fr\xE5ga"}},variables:{"ad-hoc-combobox":{"aria-label-edit-filter-operator":"Redigera filteroperator"},"ad-hoc-filter-builder":{"aria-label-add-filter":"L\xE4gg till filter","title-add-filter":"L\xE4gg till filter"},"ad-hoc-filter-renderer":{"aria-label-remove-filter":"Ta bort filter","key-select":{"placeholder-select-label":"V\xE4lj etikett"},"label-select-label":"V\xE4lj etikett","title-remove-filter":"Ta bort filter","value-select":{"placeholder-select-value":"V\xE4lj v\xE4rde"}},"data-source-variable":{label:{default:"standard"}},"default-group-by-custom-indicator-container":{"aria-label-clear":"radera",tooltip:"Till\xE4mpas som standard i denna instrumentpanel. Om den redigeras \xF6verf\xF6rs det till andra instrumentpaneler.","tooltip-restore-groupby-set-by-this-dashboard":"\xC5terst\xE4ll gruppering som inst\xE4llts av denna panel."},"format-registry":{formats:{description:{"commaseparated-values":"Kommaavgr\xE4nsade v\xE4rden","double-quoted-values":"Dubbelciterade v\xE4rden","format-date-in-different-ways":"Formatera datum p\xE5 olika s\xE4tt","format-multivalued-variables-using-syntax-example":"Formatera flerv\xE4rdesvariabler med globsyntax, till exempel {value1,value2}","html-escaping-of-values":"HTML-undantagstecken f\xF6r v\xE4rden","join-values-with-a-comma":"","json-stringify-value":"JSON stringify-v\xE4rde","keep-value-as-is":"Beh\xE5ll v\xE4rdet som det \xE4r","multiple-values-are-formatted-like-variablevalue":"Flera v\xE4rden formateras som variabel=v\xE4rde","single-quoted-values":"Enkla citerade v\xE4rden","useful-escaping-values-taking-syntax-characters":"Anv\xE4ndbart f\xF6r URL-undantagna v\xE4rden, med h\xE4nsyn till URI-syntaxtecken","useful-for-url-escaping-values":"Anv\xE4ndbart f\xF6r URL-undantagning av v\xE4rden","values-are-separated-by-character":"V\xE4rdena \xE4r \xE5tskilda med tecknet |"}}},"group-by-variable-renderer":{"aria-label-group-by-selector":"Gruppera efter v\xE4ljare","placeholder-group-by-label":"Gruppera efter etikett"},"interval-variable":{"placeholder-select-value":"V\xE4lj v\xE4rde"},"loading-options-placeholder":{"loading-options":"Laddar alternativ \u2026"},"multi-value-apply-button":{apply:"Till\xE4mpa"},"no-options-placeholder":{"no-options-found":"Inga alternativ hittades"},"options-error-placeholder":{"error-occurred-fetching-labels-click-retry":"Ett fel uppstod vid h\xE4mtning av etiketter. Klicka f\xF6r att f\xF6rs\xF6ka igen"},"test-object-with-variable-dependency":{title:{hello:"Hej"}},"test-variable":{text:{text:"Text"}},"variable-value-input":{"placeholder-enter-value":"Ange ett v\xE4rde"},"variable-value-select":{"placeholder-select-value":"V\xE4lj v\xE4rde"}}}}}}]);

//# sourceMappingURL=221.ecfe8a1e4a051cb7d19e.js.map