"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[2790],{12790(s,e,r){r.d(e,{SeriesToRowsTransformerEditor:()=>n});const n=({input:o,options:a,onChange:u})=>null}}]);

//# sourceMappingURL=2790-react19.a160414949db47d98d77.js.map