"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[3720],{43720(et,P,H){H.d(P,{DM:()=>Y,_A:()=>l,az:()=>z,pn:()=>Q});var j=H(90365);let U=0;class m{constructor(t,a,n,i){this.name=t,this.set=a,this.base=n,this.modified=i,this.id=U++}toString(){let{name:t}=this;for(let a of this.modified)a.name&&(t=`${a.name}(${t})`);return t}static define(t,a){let n=typeof t=="string"?t:"?";if(t instanceof m&&(a=t),a?.base)throw new Error("Can not derive from a modified tag");let i=new m(n,[],null,[]);if(i.set.push(i),a)for(let s of a.set)i.set.push(s);return i}static defineModifier(t){let a=new R(t);return n=>n.modified.indexOf(a)>-1?n:R.get(n.base||n,n.modified.concat(a).sort((i,s)=>i.id-s.id))}}let V=0;class R{constructor(t){this.name=t,this.instances=[],this.id=V++}static get(t,a){if(!a.length)return t;let n=a[0].instances.find(r=>r.base==t&&W(a,r.modified));if(n)return n;let i=[],s=new m(t.name,i,t,a);for(let r of a)r.instances.push(s);let f=G(a);for(let r of t.set)if(!r.modified.length)for(let d of f)i.push(R.get(r,d));return s}}function W(o,t){return o.length==t.length&&o.every((a,n)=>a==t[n])}function G(o){let t=[[]];for(let a=0;a<o.length;a++)for(let n=0,i=t.length;n<i;n++)t.push(t[n].concat(o[a]));return t.sort((a,n)=>n.length-a.length)}function Q(o){let t=Object.create(null);for(let a in o){let n=o[a];Array.isArray(n)||(n=[n]);for(let i of a.split(" "))if(i){let s=[],f=2,r=i;for(let g=0;;){if(r=="..."&&g>0&&g+3==i.length){f=1;break}let h=/^"(?:[^"\\]|\\.)*?"|[^\/!]+/.exec(r);if(!h)throw new RangeError("Invalid path: "+i);if(s.push(h[0]=="*"?"":h[0][0]=='"'?JSON.parse(h[0]):h[0]),g+=h[0].length,g==i.length)break;let p=i[g++];if(g==i.length&&p=="!"){f=0;break}if(p!="/")throw new RangeError("Invalid path: "+i);r=i.slice(g)}let d=s.length-1,c=s[d];if(!c)throw new RangeError("Invalid path: "+i);let b=new M(n,f,d>0?s.slice(0,d):null);t[c]=b.sort(t[c])}}return q.add(t)}const q=new j.uY({combine(o,t){let a,n,i;for(;o||t;){if(!o||t&&o.depth>=t.depth?(i=t,t=t.next):(i=o,o=o.next),a&&a.mode==i.mode&&!i.context&&!a.context)continue;let s=new M(i.tags,i.mode,i.context);a?a.next=s:n=s,a=s}return n}});class M{constructor(t,a,n,i){this.tags=t,this.mode=a,this.context=n,this.next=i}get opaque(){return this.mode==0}get inherit(){return this.mode==1}sort(t){return!t||t.depth<this.depth?(this.next=t,this):(t.next=this.sort(t.next),t)}get depth(){return this.context?this.context.length:0}}M.empty=new M([],2,null);function z(o,t){let a=Object.create(null);for(let s of o)if(!Array.isArray(s.tag))a[s.tag.id]=s.class;else for(let f of s.tag)a[f.id]=s.class;let{scope:n,all:i=null}=t||{};return{style:s=>{let f=i;for(let r of s)for(let d of r.set){let c=a[d.id];if(c){f=f?f+" "+c:c;break}}return f},scope:n}}function X(o,t){let a=null;for(let n of o){let i=n.style(t);i&&(a=a?a+" "+i:i)}return a}function Y(o,t,a,n=0,i=o.length){let s=new Z(n,Array.isArray(t)?t:[t],a);s.highlightRange(o.cursor(),n,i,"",s.highlighters),s.flush(i)}function at(o,t,a,n,i,s=0,f=o.length){let r=s;function d(c,b){if(!(c<=r)){for(let g=o.slice(r,c),h=0;;){let p=g.indexOf(`
`,h),E=p<0?g.length:p;if(E>h&&n(g.slice(h,E),b),p<0)break;i(),h=p+1}r=c}}Y(t,a,(c,b,g)=>{d(c,""),d(b,g)},s,f),d(f,"")}class Z{constructor(t,a,n){this.at=t,this.highlighters=a,this.span=n,this.class=""}startSpan(t,a){a!=this.class&&(this.flush(t),t>this.at&&(this.at=t),this.class=a)}flush(t){t>this.at&&this.class&&this.span(this.at,t,this.class)}highlightRange(t,a,n,i,s){let{type:f,from:r,to:d}=t;if(r>=n||d<=a)return;f.isTop&&(s=this.highlighters.filter(p=>!p.scope||p.scope(f)));let c=i,b=tt(t)||M.empty,g=X(s,b.tags);if(g&&(c&&(c+=" "),c+=g,b.mode==1&&(i+=(i?" ":"")+g)),this.startSpan(Math.max(a,r),c),b.opaque)return;let h=t.tree&&t.tree.prop(j.uY.mounted);if(h&&h.overlay){let p=t.node.enter(h.overlay[0].from+r,1),E=this.highlighters.filter(v=>!v.scope||v.scope(h.tree.type)),J=t.firstChild();for(let v=0,S=r;;v++){let _=v<h.overlay.length?h.overlay[v]:null,K=_?_.from+r:d,L=Math.max(a,S),I=Math.min(n,K);if(L<I&&J)for(;t.from<I&&(this.highlightRange(t,L,I,i,s),this.startSpan(Math.min(I,t.to),c),!(t.to>=K||!t.nextSibling())););if(!_||K>n)break;S=_.to+r,S>a&&(this.highlightRange(p.cursor(),Math.max(a,_.from+r),Math.min(n,S),"",E),this.startSpan(Math.min(n,S),c))}J&&t.parent()}else if(t.firstChild()){h&&(i="");do if(!(t.to<=a)){if(t.from>=n)break;this.highlightRange(t,a,n,i,s),this.startSpan(Math.min(n,t.to),c)}while(t.nextSibling());t.parent()}}}function tt(o){let t=o.type.prop(q);for(;t&&t.context&&!o.matchContext(t.context);)t=t.next;return t||null}const e=m.define,A=e(),x=e(),$=e(x),F=e(x),w=e(),C=e(w),B=e(w),u=e(),N=e(u),k=e(),y=e(),D=e(),O=e(D),T=e(),l={comment:A,lineComment:e(A),blockComment:e(A),docComment:e(A),name:x,variableName:e(x),typeName:$,tagName:e($),propertyName:F,attributeName:e(F),className:e(x),labelName:e(x),namespace:e(x),macroName:e(x),literal:w,string:C,docString:e(C),character:e(C),attributeValue:e(C),number:B,integer:e(B),float:e(B),bool:e(w),regexp:e(w),escape:e(w),color:e(w),url:e(w),keyword:k,self:e(k),null:e(k),atom:e(k),unit:e(k),modifier:e(k),operatorKeyword:e(k),controlKeyword:e(k),definitionKeyword:e(k),moduleKeyword:e(k),operator:y,derefOperator:e(y),arithmeticOperator:e(y),logicOperator:e(y),bitwiseOperator:e(y),compareOperator:e(y),updateOperator:e(y),definitionOperator:e(y),typeOperator:e(y),controlOperator:e(y),punctuation:D,separator:e(D),bracket:O,angleBracket:e(O),squareBracket:e(O),paren:e(O),brace:e(O),content:u,heading:N,heading1:e(N),heading2:e(N),heading3:e(N),heading4:e(N),heading5:e(N),heading6:e(N),contentSeparator:e(u),list:e(u),quote:e(u),emphasis:e(u),strong:e(u),link:e(u),monospace:e(u),strikethrough:e(u),inserted:e(),deleted:e(),changed:e(),invalid:e(),meta:T,documentMeta:e(T),annotation:e(T),processingInstruction:e(T),definition:m.defineModifier("definition"),constant:m.defineModifier("constant"),function:m.defineModifier("function"),standard:m.defineModifier("standard"),local:m.defineModifier("local"),special:m.defineModifier("special")};for(let o in l){let t=l[o];t instanceof m&&(t.name=o)}const it=z([{tag:l.link,class:"tok-link"},{tag:l.heading,class:"tok-heading"},{tag:l.emphasis,class:"tok-emphasis"},{tag:l.strong,class:"tok-strong"},{tag:l.keyword,class:"tok-keyword"},{tag:l.atom,class:"tok-atom"},{tag:l.bool,class:"tok-bool"},{tag:l.url,class:"tok-url"},{tag:l.labelName,class:"tok-labelName"},{tag:l.inserted,class:"tok-inserted"},{tag:l.deleted,class:"tok-deleted"},{tag:l.literal,class:"tok-literal"},{tag:l.string,class:"tok-string"},{tag:l.number,class:"tok-number"},{tag:[l.regexp,l.escape,l.special(l.string)],class:"tok-string2"},{tag:l.variableName,class:"tok-variableName"},{tag:l.local(l.variableName),class:"tok-variableName tok-local"},{tag:l.definition(l.variableName),class:"tok-variableName tok-definition"},{tag:l.special(l.variableName),class:"tok-variableName2"},{tag:l.definition(l.propertyName),class:"tok-propertyName tok-definition"},{tag:l.typeName,class:"tok-typeName"},{tag:l.namespace,class:"tok-namespace"},{tag:l.className,class:"tok-className"},{tag:l.macroName,class:"tok-macroName"},{tag:l.propertyName,class:"tok-propertyName"},{tag:l.operator,class:"tok-operator"},{tag:l.comment,class:"tok-comment"},{tag:l.meta,class:"tok-meta"},{tag:l.invalid,class:"tok-invalid"},{tag:l.punctuation,class:"tok-punctuation"}])}}]);

//# sourceMappingURL=3720-react19.8c1bc088838d84244b39.js.map