"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[1157],{28211(i,r,n){n.r(r),n.d(r,{default:()=>U});var a=n(14553),s=n(55501),l=n(81664),P=n(16619),D=n(98391),e=n(5989),d=n(13400);const E=(0,s.lazy)(()=>Promise.all([n.e(3809),n.e(6813),n.e(8348),n.e(9136)]).then(n.bind(n,26506))),O=(0,s.lazy)(()=>n.e(703).then(n.bind(n,83055)));function h(t){return(0,e.YN)({suspend:!0})?(0,a.jsx)(M,{...t}):(0,a.jsx)(E,{...t})}function M(t){const{data:o,isLoading:g,isError:m}=(0,l.Mu)(),f=o?.spec?.homeURL;return(0,s.useEffect)(()=>{if(!f)return;const u=P.I.processRedirectUri(f,D.Ny.getLocation());D.Ny.replace(u)},[f]),g||f?(0,a.jsx)(d._,{text:""}):m||!o?(0,a.jsx)(E,{...t}):o.spec?.homeDashboardUID?(0,a.jsx)(E,{...t}):(0,a.jsx)(O,{})}function U(t){return(0,a.jsx)(s.Suspense,{fallback:(0,a.jsx)(d._,{text:""}),children:(0,a.jsx)(h,{...t})})}}}]);

//# sourceMappingURL=HomeRoute-react19.d33400be9700787a54a9.js.map