"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[6888],{76888(v,l,a){a.d(l,{LimitTransformerEditor:()=>d});var r=a(14553),n=a(55501),_=a(52358),E=a(6634),o=a(86979),s=a(75251);const d=({options:t,onChange:e})=>{const[u,g]=(0,n.useState)(!1),m=(0,n.useCallback)(i=>{g(!(0,s.N_)(i)),e({...t,limitField:i})},[e,t]);return(0,r.jsx)(r.Fragment,{children:(0,r.jsx)(E.C,{children:(0,r.jsx)(o.y,{invalid:u,error:"Value needs to be an integer or a variable",value:String(t.limitField),onChange:m,placeholder:(0,_.t)("transformers.limit-transformer-editor.placeholder-value-or-variable","Value or variable"),suggestions:(0,s.I9)()})})})}}}]);

//# sourceMappingURL=6888-react19.a3324bc7b03794fb5fbb.js.map